import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { catchError } from 'rxjs/operators';
import { AuthErrorMessages } from 'src/app/models/auth/auth-error-messages.enum';
import { AuthUserDTO } from 'src/app/models/auth/auth-user.dto';
import { UpdateUserDTO } from 'src/app/models/auth/update-user.dto';
import { ErrorDTO } from 'src/app/models/error/error-dto';
import { ServiceErrorCode } from 'src/app/models/error/service-error-code.enum';
import { AuthService } from 'src/app/services/auth/auth.service';
import { SpinnerService } from 'src/app/services/spinner/spinner.service';
import { ThemeService } from 'src/app/services/theme/theme.service';
import { ToastService } from 'src/app/services/toast/toast.service';

@Component({
  selector: 'app-user-settings',
  templateUrl: './user-settings.component.html',
  styleUrls: ['./user-settings.component.scss']
})
export class UserSettingsComponent implements OnInit {
  settingsForm!: FormGroup<{
    firstName: FormControl<string | null>,
    lastName: FormControl<string | null>,
    email: FormControl<string | null>,
    password: FormControl<string | null>
  }>;
  submitted = false;
  ready = false;
  errors = {
    email: false,
    password: false
  };

  authUser: AuthUserDTO | null = null;

  constructor(
    private authService: AuthService,
    private formBuilder: FormBuilder,
    private router: Router,
    private spinnerService: SpinnerService,
    private toastService: ToastService,
    private themeService: ThemeService,
    private translaste: TranslateService
  ) { }

  ngOnInit(): void {
    this.initForms();
    this.load();
  }

  initForms(): void {
    this.settingsForm = this.formBuilder.group({
      firstName: ['', [Validators.required]],
      lastName: ['', [Validators.required]],
      email: ['', [Validators.required, Validators.email]],
      password: ['', [Validators.required, Validators.minLength(8)]]
    });
  }

  resetErrorFlags(): void {
    this.errors = {
      email: false,
      password: false
    };
  }

  load(): void {
    const token = this.authService.getDecodedToken();
    const mail = this.authService.getEMailFromDecodedToken(token);
    if (!mail) {
      return;
    }

    this.authService.findUserByEMail(mail)
      .pipe(catchError(err => {
        this.toastService.error(this.translaste.instant('view.user_settings.message.user_not_found'), this.translaste.instant('view.user_settings.message.user_not_found_d'));
        this.authService.logout();
        throw err;
      }))
      .subscribe(user => {
        if (!user) {
          this.toastService.error(this.translaste.instant('view.user_settings.message.user_not_found'), this.translaste.instant('view.user_settings.message.user_not_found_d'));
          this.authService.logout();
        }
        this.authUser = user;
        this.settingsForm.controls.firstName.setValue(user.firstName);
        this.settingsForm.controls.lastName.setValue(user.lastName);
        this.settingsForm.controls.email.setValue(user.email);
      });
  }

  saveSettings(): void {
    if (!this.authUser) {
      return;
    }

    this.resetErrorFlags();
    this.submitted = true;
    this.spinnerService.showSpinner();

    this.authUser.password = this.settingsForm.value.password ?? null;
    const updateUserDTO: UpdateUserDTO = {
      authUserDTO: this.authUser,
      newAuthUserDTO: {
        firstName: this.settingsForm.value.firstName ?? null,
        lastName: this.settingsForm.value.lastName ?? null,
        email: this.settingsForm.value.email ?? null,
        password: null
      }
    };

    this.authService.updateUser(updateUserDTO)
      .pipe(catchError(error => {
        if (error.error !== null) {
          const err: ErrorDTO = error.error;

          if (err.errorCode === ServiceErrorCode.InvalidUser && err.message === AuthErrorMessages.WrongPassword) {
            this.errors.password = true;
          } else if (err.errorCode === ServiceErrorCode.InvalidUser && err.message === AuthErrorMessages.UserAlreadyExists) {
            this.errors.email = true;
          }
        }
        this.spinnerService.hideSpinner();
        this.toastService.error(this.translaste.instant('view.user_settings.message.error'), this.translaste.instant('view.user_settings.message.could_not_change_settings'));
        throw error;
      }))
      .subscribe(resp => {
        updateUserDTO.newAuthUserDTO.password = updateUserDTO.authUserDTO.password;
        this.authService.login(updateUserDTO.newAuthUserDTO)
          .pipe(catchError(err => {
            this.router.navigate(['/auth/login']);
            throw err;
          }))
          .subscribe(token => {
            this.spinnerService.hideSpinner();
            if (token) {
              this.toastService.success(this.translaste.instant('view.user_settings.message.success'), this.translaste.instant('view.user_settings.message.changed_settings'));
              this.authService.saveToken(token);
              this.themeService.loadTheme();
              this.themeService.loadMenu();
              this.load();
              return true;
            }
            this.router.navigate(['/auth/login']);
            return false;
          });
      });
  }

}
